﻿using BH.Signup.Core.Domain;
using Microsoft.AspNetCore.Diagnostics;
using Microsoft.AspNetCore.Mvc;
using System.Threading.Tasks;
using System.Reflection;
using BH.Signup.Core.Models;

namespace Signup.Billhighway.Com.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class ErrorsController : Controller
    {
        #region Constructors & Initialization

        /// <summary>
        /// Initializes a new instance of the <see cref="ErrorsController"/> class.
        /// </summary>
        /// <param name="logService">The log service.</param>
        public ErrorsController(ILogService logService)
        {
            LogService = logService;
        }

        #endregion

        #region Public Properties

        /// <summary>Gets the log service.</summary>
        /// <value>The log service.</value>
        public ILogService LogService { get; private set; }

        #endregion

        /// <summary>Errors this instance.</summary>
        /// <returns></returns>
        public async Task<IActionResult> Error(ErrorModel incomingModel)
        {
            var error = this.HttpContext.Features.Get<IExceptionHandlerFeature>();
            int errorID = 0;


            if (error != null)
            {
                errorID = (await LogService.Log(new LogRequest
                {
                    Exception = error.Error,
                    Comment = "",
                    Method = this.GetType().GetMethod(nameof(Error)),
                    Path = "/Error",
                    UserID = 0
                }));
            }

            var model = incomingModel ?? new ErrorModel();
            model.RTEID = errorID;


            return View(model);
        }
    }
}
